#!/bin/sh
#\
exec wish "$0" "$@"

############## Variables ###################

proc set_tcgui {ix lst} {
	global tcgui
	foreach {name val} $lst {
		set tcgui($ix,$name) $val
	}
}

set_tcgui cmds { DISPLAY 1 WITHDRAW 2 SET-ATTR 3 INSERT-VALUE 4 
	SELECT-VALUE 5 WAIT 6}
set_tcgui events { READY 0 ERROR 1 CLOSE 2 }
set_tcgui attrs { FOREGROUND 1 BACKGROUND 2 FONT 3 RELIEF 4 BORDER 5 }

set tcgui(name) "EXAMPLE"
set tcgui(defaults) { }
set tcgui(autoname) 0
set tcgui(snap) 5
set tcgui(widgets) {}
set tcgui(container) .gui

set comments(1) \
{************************************************************************
*
* Graphical User Interface generated by TCGUI
* 
* at date/time [clock format [clock seconds]] 
* from file $fn
*
************************************************************************}

############## Builder interface ##################
. config -menu .m
pack [frame .prop -width 450 -height 300] -expand 1 -fill both
wm title . "TCGUI builder"
wm minsize . 300 150

menu .m
.m add cascade -menu .m.file  -label File -underline 0
.m add cascade -menu .m.help  -label Help -underline 0

menu .m.file
.m.file add command -label New -underline 0 -command file_new 
.m.file add command -label Save -underline 0 -command file_save 
.m.file add command -label Load -underline 0 -command file_load
.m.file add command -label Generate -underline 0 -command generate_cpy 
.m.file add separator
.m.file add command -label exit -command exit -underline 1

menu .m.help
.m.help add command -label About

################## BINDINGS ######################

bind . <Control-c> exit

############ ENCODING AND FILE MANIPULATION  ##############
proc set_mainwin {appl mw mh} {
	global tcgui
	set tcgui(name) $appl
	.gui config -width $mw -height $mh
}

proc set_coboffset {} {
	global tcgui
	set ofs 0
	set radios {}
	set offsets {}
	foreach w $tcgui(widgets) {
		set w $w.w
		if {[info exists tcgui(value,$w,cobolvar)] &&
			$tcgui(value,$w,cobolvar) != "" &&
			[info exists tcgui(value,$w,cobsize)] &&
			$tcgui(value,$w,cobsize) != ""} {
				set tcgui(coboffset,$w) $ofs
				if {[winfo class $w] != "Radiobutton"} {
					incr ofs $tcgui(value,$w,cobsize)
				} elseif {[lsearch $radios $tcgui(value,$w,cobolvar)] < 0} {
					lappend radios $tcgui(value,$w,cobolvar)
					lappend offsets $ofs
					incr ofs
				} else {
					set n [lsearch $radios $tcgui(value,$w,cobolvar)]
					set tcgui(coboffset,$w) [lindex $offsets $n]
				}
		}
	}
}

proc file_new {} {
	global tcgui
	foreach w $tcgui(widgets) {
		#if {$tcgui(parent,$w) == ".gui"} 
		if {$tcgui(parent,$w) == "."} {
			destroy $w
		}
	}
	set tcgui(widgets) {}
	set tcgui(autoname) 0
}

proc file_save {} {
	global tcgui
	set types { {{design files} {.gui}} {{All files} {*}} }
	set initdir ""
	if {[info exists tcgui(initialdir)]} {
		set initdir "-initialdir $tcgui(initialdir)"
	}
	set initfile ""
	if {[info exists tcgui(initialfile)]} {
		set initdir "-initialfile $tcgui(initialfile)"
	}
	set fn [eval tk_getSaveFile -defaultextension .gui -filetypes \$types \
		$initdir $initfile]
	if {$fn == ""} {
		return ""
	}
	set tcgui(initialdir) [file dirname $fn]
	set tcgui(initialfile) [file tail $fn]
	
	set_coboffset
	set f [open $fn w]
	set mw [winfo width .gui] 
	set mh [winfo height .gui]
	puts $f "set_mainwin $tcgui(name) $mw $mh"
	foreach w $tcgui(widgets) {
		set parent $tcgui(parent,$w)
		set cmd $tcgui(ccmd,[winfo class $w.w])
		set name [lindex [split $w .] end]
		array set p [place info $w]
		set width [lindex [$w config -width] 4]
		set height [lindex [$w config -height] 4]
		set ats {}
		set ww $w.w
		foreach at $tcgui(attrib,[winfo class $ww]) {
			if {[info exists tcgui(value,$ww,$at)] &&
				$tcgui(value,$ww,$at) != "" &&
				[lsearch {cobolvar cobsize coboffset} $at] < 0 } {
					lappend ats $at $tcgui(value,$ww,$at)
			}
		}
		set cobolvar "\{\}"
		set cobsize 1
		set coboffset 0
		if {[info exists tcgui(value,$ww,cobolvar)] &&
			$tcgui(value,$ww,cobolvar) != ""} {
				set cobolvar $tcgui(value,$ww,cobolvar)
			if {[info exists tcgui(value,$ww,cobsize)] &&
				$tcgui(value,$ww,cobsize) != ""} {
					set cobsize $tcgui(value,$ww,cobsize)
			}
			if {[info exists tcgui(coboffset,$ww)] &&
				$tcgui(coboffset,$ww) != ""} {
					set coboffset $tcgui(coboffset,$ww)
			}
		}
		set cmdline "$cmd $name $parent $p(-x) $p(-y)"
		append cmdline " $width $height $cobolvar $cobsize $coboffset $ats"
		puts $f $cmdline
	}
	close $f
	return $fn
}

proc file_load {} {
	global tcgui
	set types { {{design files} {.gui}} {{All files} {*}} }
	set initdir ""
	if {[info exists tcgui(initialdir)]} {
		set initdir "-initialdir $tcgui(initialdir)"
	}
	set initfile ""
	if {[info exists tcgui(initialfile)]} {
		set initdir "-initialfile $tcgui(initialfile)"
	}
	set fn [eval tk_getOpenFile -filetypes \$types \
		$initdir $initfile]
	if {$fn == ""} {
		return ""
	}
	set tcgui(initialdir) [file dirname $fn]
	set tcgui(initialfile) [file tail $fn]
	
	file_new
	source $fn
	set i 0
	foreach w $tcgui(widgets) {
		set name [lindex [split $w .] end]
		catch {set i [expr [string trimleft $name 0]+1]}
		if {$i > $tcgui(autoname)} {
			set tcgui(autoname) $i
		}
	}
}

proc puts_cmds {f} {
	global tcgui
	foreach {ix val} [array get tcgui cmds,*] {
		set n [lindex [split $ix ,] end]
		puts $f "     88  $tcgui(name)-CMD-$n VALUE $val."
	}
}

proc puts_events {f} {
	global tcgui
	foreach {ix val}  [array get tcgui events,*] {
		set n [lindex [split $ix ,] end]
		puts $f "     88  $tcgui(name)-SYSEV-$n VALUE $val."
	}
	# generate user button events (1-10 system events, reserved)
	set usr_ev 10
	foreach w $tcgui(widgets) {
	    if {[info exists tcgui(value,$w.w,eventname)]} {
			set evname $tcgui(value,$w.w,eventname)
			puts $f "     88  $tcgui(name)-EV-$evname VALUE [incr usr_ev]."
		}
	}
}

proc puts_attrs {f} {
	global tcgui
	foreach {ix val} [array get tcgui attrs,*] {
		set n [lindex [split $ix ,] end]
		puts $f "     88  $tcgui(name)-ATTR-$n VALUE $val."
	}
}

proc puts_widgets {f} {
	global tcgui
	foreach w $tcgui(widgets) {
		set n [lindex [split $w ,] end]
		set n1 [lindex [split $n .] end]
		set wname $n1
		if {[info exists tcgui(value,$w.w,widgetname)] &&
			$tcgui(value,$w.w,widgetname) != ""} {
			 set wname $tcgui(value,$w.w,widgetname)
		}	
		puts $f \
		"     88  $tcgui(name)-WIDGET-$wname VALUE [string trimleft $n1 0]."
	}
}

proc puts_cobolvars {f} {
	global tcgui
	set varlist {}
	set size 0
	foreach w $tcgui(widgets) {
		set w $w.w
		if {[info exists tcgui(value,$w,cobolvar)] &&
			$tcgui(value,$w,cobolvar) != ""} {
				set v $tcgui(value,$w,cobolvar)
				if {[lsearch $varlist $v] < 0} {
					lappend varlist $v
					if {[winfo class $w] == "Radiobutton"} {
						incr size
					}
				}
				set vsize($v) 0
				if {[winfo class $w] == "Radiobutton"} {
					set vsize($v) 1
					lappend vlist($v) $w
				} elseif {[winfo class $w] == "Checkbutton"} {
					set vsize($v) 1
					incr size $vsize($v)
				} else {
					set vsize($v) $tcgui(value,$w,cobsize)
					incr size $vsize($v)
				}
				set vw($v) $w
		}
	}
#
	puts $f "   05    $tcgui(name)-COBOL-VARS-SIZE PIC 9(9) COMP VALUE $size."
	if {$size} {
		puts $f "   05    $tcgui(name)-COBOL-VARIABLES."
	}
	foreach v $varlist {
		if {[winfo class $vw($v)] == "Radiobutton"} {
			puts $f "     10  $v PIC X."
			foreach vl $vlist($v) {
				set n1 [lindex [split $vl .] end-1]
				set wname $n1
				if {[info exists tcgui(value,$vl,widgetname)] &&
						$tcgui(value,$vl,widgetname) != ""} {
						set wname $tcgui(value,$vl,widgetname)
				}	
				set selval $tcgui(value,$vl,selval)
				puts $f "       88  $wname VALUE \"$selval\"."
			}
		} elseif {[winfo class $vw($v)] == "Checkbutton"} {
				puts $f "     10  $v PIC 9."
				puts $f "       88  $v-ON VALUE 1."
				puts $f "       88  $v-OFF VALUE 0."
		} else {
			puts $f "     10  $v PIC X($vsize($v))."
		}
	}
}

proc generate_cpy {} {
	global tcgui comments
	set fn [file_save]
# compress with gzip
	catch {file delete tmp.tcl.gz}
	exec gzip -c [file rootname $fn].gui > tmp.tcl.gz

# encode file (base64)
	set f [open tmp.tcl.gz r]
	fconfigure $f -encoding binary -translation binary
	set s [read $f]
	close $f
	file delete tmp.tcl.gz

# save copybook
	set f [open [file rootname $fn].cpy w]
	puts $f [subst $comments(1)]
	puts $f " 01      $tcgui(name)-GUI."
	puts $f "   05    $tcgui(name)-GUI-COMMAND PIC 9(9) COMP."
	puts_cmds $f
	puts $f "   05    $tcgui(name)-GUI-EVENT PIC 9(9) COMP."
	puts_events $f
	puts $f "   05    $tcgui(name)-GUI-NAME PIC X(40) VALUE \"$tcgui(name)\"."
	puts $f "   05    $tcgui(name)-ATTRIB-TYPE PIC 9(9) COMP."
	puts_attrs $f
	puts $f "   05    $tcgui(name)-ATTRIB-VALUE PIC X(80)."
	puts $f "   05    $tcgui(name)-WIDGET-SELECT PIC 9(9) COMP."
	puts_widgets $f
	puts_cobolvars $f
	puts $f "   05    $tcgui(name)-GUI-LOADED PIC 9(9) COMP VALUE 0."
	puts $f "   05    $tcgui(name)-GUI-CODE."
	foreach line [split [encode $s] \n] {
		puts $f "    10   FILLER PIC X(60) VALUE\n         \"$line\"."
	}
	close $f
}

