#!/bin/sh
# 
# Convert a tcl script into a C string to be #included 
# 
#\
exec tclsh "$0" "$@"

if {$argc < 3} {
	puts "usage: $argv0 file.tcl file.c stringname"
	exit 1
}

set source  [lindex $argv 0]
set output  [lindex $argv 1]
set strname [lindex $argv 2]


proc makeSubs {line} {
	global encondings strname
	upvar $line ln
	# substitute \ for something unused to avoid confusing further subst
	regsub -all -- {\\} $ln \01 ln
	regsub -all -- {\"} $ln \\\" ln
# "  <-- this comment removes a bug in vim colorizing!
	regsub -all -- {\01} $ln {\\\\} ln
}

set inf [open $source r]
set outf [open $output w]
puts $outf "char *$strname[] = {"
while {![eof $inf]} {
	gets $inf line
	makeSubs line
	if {($line == "") || [regexp {^#} $line]} {
		continue
	}
	puts $outf "\"$line\\n\""
}
puts $outf "};"
close $outf
close $inf

